@@ -1,4 +1,6 @@
-####### Notes #########
+
+# General notes -----------------------------------------------------------
+
 
 # This file contains the wrappers for the models supported by DHARMa. DHARMa is interaction with packages ONLY via the wrappers. Below are
 #
@@ -14,21 +16,40 @@
 
 # See comments in the help of the generic S3 functions for guidance about how to implement each function
 
-####### New Class Template #########
 
-# getObservedResponse
+# Checks -----------------------------------------------------------
 
-# getSimulations
-
-# getFixedEffects
-
-# getRefit
-
-# getFitted
-
-# getResiduals
-
-######### Generics #############
+#' Check if the fitted model is supported by DHARMa
+#'
+#' The function checks if the fitted model is supported by DHARMa, and if there are other issues that could create problems
+#'
+#' @param fittedModel a fitted model
+#' @param stop whether to throw an error if the model is not supported by DHARMa
+#'
+#' @details The main purpose of this function os to check if the fitted model class is supported by DHARMa. The function additionally checks for properties of the fitted model that could create problems for calculating residuals or working with the resuls in DHARMa.
+#'
+#'
+#' @keywords internal
+checkModel <- function(fittedModel, stop = F){
+  
+  out = T
+  
+  if(!(class(fittedModel)[1] %in% getPossibleModels())){
+    if(stop == FALSE) warning("DHARMa: fittedModel not in class of supported models. Absolutely no guarantee that this will work!")
+    else stop("DHARMa: fittedModel not in class of supported models")
+  }
+  
+  # if(hasNA(fittedModel)) message("It seems there were NA values in the data used for fitting the model. This can create problems if you supply additional data to DHARMa functions. See ?checkModel for details")
+  
+  # TODO: check as implemented does not work reliably, check if there is any other option to check for NA
+  # #' @example inst/examples/checkModelHelp.R
+  
+  #  NA values in the data: checkModel will detect if there were NA values in the data frame. For NA values, most regression models will remove the entire observation from the data. This is not a problem for DHARMa - residuals are then only calculated for non-NA rows in the data. However, if you provide additional predictors to DHARMa, for example to plot residuals against a predictor, you will have to remove all NA rows that were also removed in the model. For most models, you can get the rows of the data that were actually used in the fit via rownames(model.frame(fittedModel))
+  
+  
+  #if (class(fittedModel)[1] == "gam" ) if (class(fittedModel$family)[1] == "extended.family") stop("It seems you are trying to fit a model from mgcv that was fit with an extended.family. Simulation functions for these families are not yet implemented in DHARMa. See issue https://github.com/florianhartig/DHARMa/issues/11 for updates about this")
+  
+}
 
 #' get possible models
 #' 
@@ -37,7 +58,11 @@
 #' @keywords internal
 getPossibleModels<-function()c("lm", "glm", "negbin", "lmerMod", "lmerModLmerTest", "glmerMod", "gam", "bam", "glmmTMB", "HLfit", "MixMod")
 
-# New S3 methods
+
+weightsWarning = "Model was fit with prior weights. These will be ignored in the simulation. See ?getSimulations for details"
+
+
+######### Generic S3 Wrappers #############
 
 #' Get model response
 #'
@@ -58,32 +83,6 @@ getObservedResponse <- function (object, ...) {
   UseMethod("getObservedResponse", object)
 }
 
-#' @rdname getObservedResponse
-#' @export
-getObservedResponse.default <- function (object, ...){
-  out = model.frame(object)[,1]
-
-  # check for weights in k/n case
-  if(family(object)$family %in% c("binomial", "betabinomial") & "(weights)" %in% colnames(model.frame(object))){
-    x = model.frame(object)
-    out = out * x$`(weights)`
-  }
-
-  # check for k/n binomial
-  if(is.matrix(out)){
-    if(!(ncol(out) == 2)) securityAssertion("nKcase - wrong dimensions of response")
-    if(!(family(object)$family %in% c("binomial", "betabinomial"))) securityAssertion("nKcase - wrong family")
-
-    out = out[,1]
-  }
-
-  # observation is factor - unlike lme4 and older, glmmTMB simulates nevertheless as numeric
-  if(is.factor(out)) out = as.numeric(out) - 1
-
-  return(out)
-}
-
-weightsWarning = "Model was fit with prior weights. These will be ignored in the simulation. See ?getSimulations for details"
 
 #' Get model simulations
 #'
@@ -115,71 +114,20 @@ getSimulations <- function (object, nsim = 1 , type = c("normal", "refit"), ...)
   UseMethod("getSimulations", object)
 }
 
-#' @rdname getSimulations
-#' @export
-getSimulations.default <- function (object, nsim = 1, type = c("normal", "refit"), ...){
-
-  type <- match.arg(type)
-
-  out = simulate(object, nsim = nsim , ...)
-
-  if (type == "normal"){
-    if(family(object)$family %in% c("binomial", "betabinomial")){
-      if("(weights)" %in% colnames(model.frame(object))){
-        x = model.frame(object)
-        out = out * x$`(weights)`
-      } else if (is.matrix(out[[1]])){
-        # this is for the k/n binomial case
-        out = as.matrix(out)[,seq(1, (2*nsim), by = 2)]
-      } else if(is.factor(out[[1]])){
-        if(nlevels(out[[1]]) != 2){
-          warning("The fitted model has a factorial response with number of levels not equal to 2 - there is currently no sensible application in DHARMa that would lead to this situation. Likely, you are trying something that doesn't work.")
-        }
-        else{
-          out = data.matrix(out) - 1
-        }
-      }
-    }
-
-    if(!is.matrix(out)) out = data.matrix(out)
-  } else{
-    if(family(object)$family %in% c("binomial", "betabinomial")){
-      if (!is.matrix(out[[1]]) & !is.numeric(out)) data.frame(data.matrix(out)-1)
-    }
-  }
-
-  return(out)
-}
-
-
 #' Extract fixed effects of a supported model
 #'
 #' A wrapper to extract fixed effects of a supported model
 #'
-#' @param fittedModel a fitted model
-#'
+#' @param object a fitted model
+#' @param ... additional parameters
 #' @example inst/examples/wrappersHelp.R
-#'
-#' @importFrom lme4 fixef
-#'
 #' @seealso \code{\link{getObservedResponse}}, \code{\link{getSimulations}}, \code{\link{getRefit}}, \code{\link{getFitted}}
 #' @export
-getFixedEffects <- function(fittedModel){
-
-  if(class(fittedModel)[1] %in% c("glm", "lm", "gam", "bam", "negbin") ){
-    out  = coef(fittedModel)
-  } else if(class(fittedModel)[1] %in% c("glmerMod", "lmerMod", "HLfit")){
-    out = fixef(fittedModel)
-  } else if(class(fittedModel)[1] %in% c("glmmTMB")){
-    out = fixef(fittedModel)
-    out = out$cond
-  } else {
-    out = coef(fittedModel)
-    if(is.null(out)) out = fixef(fittedModel)
-  }
-  return(out)
+getFixedEffects <- function(object, ...){
+  UseMethod("getFixedEffects", object)
 }
 
+
 #' Get model refit
 #'
 #' Wrapper to refit a fitted model
@@ -199,15 +147,6 @@ getRefit <- function (object, newresp, ...) {
   UseMethod("getRefit", object)
 }
 
-#' @rdname getRefit
-#'
-#' @importFrom lme4 refit
-#'
-#' @export
-getRefit.default <- function (object, newresp, ...){
-  refit(object, newresp, ...)
-}
-
 #' Get fitted / predicted values
 #'
 #' Wrapper to get the fitted / predicted response of model at the response scale
@@ -231,13 +170,6 @@ getFitted <- function (object, ...) {
 
 # NOTE - a bit unclear if fitted or predict should be used
 
-#' @rdname getFitted
-#' @export
-getFitted.default <- function (object,...){
-  out = predict(object, type = "response", re.form = ~0)
-  out = as.vector(out) # introduced because of phyr error
-}
-
 
 #' Get model residuals
 #'
@@ -260,6 +192,106 @@ getResiduals <- function (object, ...) {
 
 # NOTE - a bit unclear if fitted or predict should be used
 
+
+
+
+# default -----------------------------------------------------------------
+
+#' @rdname getObservedResponse
+#' @export
+getObservedResponse.default <- function (object, ...){
+  out = model.frame(object)[,1]
+  
+  # check for weights in k/n case
+  if(family(object)$family %in% c("binomial", "betabinomial") & "(weights)" %in% colnames(model.frame(object))){
+    x = model.frame(object)
+    out = out * x$`(weights)`
+  }
+  
+  # check for k/n binomial
+  if(is.matrix(out)){
+    if(!(ncol(out) == 2)) securityAssertion("nKcase - wrong dimensions of response")
+    if(!(family(object)$family %in% c("binomial", "betabinomial"))) securityAssertion("nKcase - wrong family")
+    
+    out = out[,1]
+  }
+  
+  # observation is factor - unlike lme4 and older, glmmTMB simulates nevertheless as numeric
+  if(is.factor(out)) out = as.numeric(out) - 1
+  
+  return(out)
+}
+
+#' @rdname getRefit
+#' @importFrom lme4 refit
+#' @export
+getRefit.default <- function (object, newresp, ...){
+  refit(object, newresp, ...)
+}
+
+#' @rdname getSimulations
+#' @export
+getSimulations.default <- function (object, nsim = 1, type = c("normal", "refit"), ...){
+  
+  type <- match.arg(type)
+  
+  out = simulate(object, nsim = nsim , ...)
+  
+  if (type == "normal"){
+    if(family(object)$family %in% c("binomial", "betabinomial")){
+      if("(weights)" %in% colnames(model.frame(object))){
+        x = model.frame(object)
+        out = out * x$`(weights)`
+      } else if (is.matrix(out[[1]])){
+        # this is for the k/n binomial case
+        out = as.matrix(out)[,seq(1, (2*nsim), by = 2)]
+      } else if(is.factor(out[[1]])){
+        if(nlevels(out[[1]]) != 2){
+          warning("The fitted model has a factorial response with number of levels not equal to 2 - there is currently no sensible application in DHARMa that would lead to this situation. Likely, you are trying something that doesn't work.")
+        }
+        else{
+          out = data.matrix(out) - 1
+        }
+      }
+    }
+    
+    if(!is.matrix(out)) out = data.matrix(out)
+  } else{
+    if(family(object)$family %in% c("binomial", "betabinomial")){
+      if (!is.matrix(out[[1]]) & !is.numeric(out)) data.frame(data.matrix(out)-1)
+    }
+  }
+  
+  return(out)
+}
+
+
+#' @rdname getFixedEffects
+#' @importFrom lme4 fixef
+#' @export
+getFixedEffects.default <- function(object, ...){
+  
+  if(class(object)[1] %in% c("glm", "lm", "gam", "bam", "negbin") ){
+    out  = coef(object)
+  } else if(class(object)[1] %in% c("glmerMod", "lmerMod", "HLfit")){
+    out = fixef(object)
+  } else if(class(object)[1] %in% c("glmmTMB")){
+    out = glmmTMB::fixef(object)
+    out = out$cond
+  } else {
+    out = coef(object)
+    if(is.null(out)) out = fixef(object)
+  }
+  return(out)
+}
+
+#' @rdname getFitted
+#' @export
+getFitted.default <- function (object,...){
+  out = predict(object, type = "response", re.form = ~0)
+  out = as.vector(out) # introduced because of phyr error
+}
+
 #' @rdname getFitted
 #' @export
 getResiduals.default <- function (object,...){
@@ -282,14 +314,16 @@ getResiduals.default <- function (object,...){
 #   else return(FALSE)
 # }
 
+
+
 ######### LM #############
 
 #' @rdname getRefit
 #' @export
 getRefit.lm <- function(object, newresp, ...){
-
+  
   newData <-model.frame(object)
-
+  
   if(is.vector(newresp)){
     newData[,1] = newresp
   } else if (is.factor(newresp)){
@@ -300,7 +334,7 @@ getRefit.lm <- function(object, newresp, ...){
     newData[[1]] = NULL
     newData = cbind(newresp, newData)
   }
-
+  
   refittedModel = update(object, data = newData)
   return(refittedModel)
 }
@@ -339,13 +373,72 @@ getFitted.gam <- function(object, ...){
 # plot(fitted(fittedModelGAM), predict(fittedModelGAM, type = "response"))
 
 
+# Get Simulations of gam object
+
+#' @rdname getSimulations
+#' @param mgcViz whether simulations should be created with mgcViz (if mgcViz is available)
+#' @export
+getSimulations.gam <- function(object, nsim = 1, type = c("normal", "refit"), mgcViz = TRUE, ...){
+  
+  type <- match.arg(type)
+  
+  if(length(find.package("mgcViz")) > 0 & mgcViz == T){
+    
+    if("(weights)" %in% colnames(model.frame(object)) & ! family(object)$family %in% c("binomial", "betabinomial")) warning(weightsWarning)
+    
+    # use mgcViz if available
+    out = mgcViz::simulate.gam(object, nsim = nsim , ...)
+    out = as.data.frame(out)
+    
+    # from here to end identical to default
+    
+    if (type == "normal"){
+      if(family(object)$family %in% c("binomial", "betabinomial")){
+        if("(weights)" %in% colnames(model.frame(object))){
+          x = model.frame(object)
+          out = out * x$`(weights)`
+        } else if (is.matrix(model.frame(object)[[1]])){
+          # this is for the k/n binomial case
+          out = out * rowSums(model.frame(object)[[1]])
+        }
+      }
+      if(!is.matrix(out)) out = data.matrix(out)
+    } else{
+      if(family(object)$family %in% c("binomial", "betabinomial")){
+        if (is.matrix(model.frame(object)[[1]])){
+
+          # need to convert to k,n-k format for refit. Syntax here is from
+          # https://stackoverflow.com/questions/6143697/data-frame-with-a-column-containing-a-matrix-in-r
+
+          trials = rowSums(model.frame(object)[[1]])
+          oldnames = names(out)
+          oldrownames = row.names(out)
+          out = out * trials
+          out = as.list(out)
+          for(i in 1:length(out)){
+            out[[i]] = cbind(out[[i]], trials - out[[i]])
+            colnames(out[[i]]) = colnames(model.frame(object)[[1]])
+          }
+          class(out) = "data.frame"
+          names(out) = oldnames
+          row.names(out) = oldrownames
+        }
+      }
+    }
+  } else {
+    message("It seems you don't have mgcViz installed on this computer. When using DHARMa with mgcv objects, it is highly recommended to also install mgcViz, which will extend the ability of DHARMa to simulate from various gam objects. Withoug mgcViz, simulations will fail in various situations. See vignette for details!")
+    out = getSimulations.default(object, nsim, type, ...)
+  }
+  return(out)
+}
+
 ######## lme4 ############
 
 
 #' @rdname getSimulations
 #' @export
 getSimulations.lmerMod <- function (object, nsim = 1, type = c("normal", "refit"), ...){
-
+  
   if("(weights)" %in% colnames(model.frame(object))) warning(weightsWarning)
   getSimulations.default(object = object, nsim = nsim, type = type, ...)
 }
@@ -357,7 +450,7 @@ getSimulations.lmerMod <- function (object, nsim = 1, type = c("normal", "refit"
 #' @export
 getRefit.glmmTMB <- function(object, newresp, ...){
   newData <-model.frame(object)
-
+  
   # hack to make update work - for some reason, glmmTMB wants the matrix embedded in the df for update to work  ... should be solved ideally, see https://github.com/glmmTMB/glmmTMB/issues/549
   if(is.matrix(newresp)){
     tmp = colnames(newData[[1]])
@@ -378,13 +471,13 @@ getRefit.glmmTMB <- function(object, newresp, ...){
 #' @rdname getSimulations
 #' @export
 getSimulations.glmmTMB <- function (object, nsim = 1, type = c("normal", "refit"), ...){
-
+  
   if("(weights)" %in% colnames(model.frame(object)) & ! family(object)$family %in% c("binomial", "betabinomial")) warning(weightsWarning)
-
+  
   type <- match.arg(type)
-
+  
   out = simulate(object, nsim = nsim, ...)
-
+  
   if (type == "normal"){
     if (is.matrix(out[[1]])){
       # this is for the k/n binomial case
@@ -392,7 +485,7 @@ getSimulations.glmmTMB <- function (object, nsim = 1, type = c("normal", "refit"
     }
     if(!is.matrix(out)) out = data.matrix(out)
   }else{
-
+    
     # check for weights in k/n case
     if(family(object)$family %in% c("binomial", "betabinomial")){
       if("(weights)" %in% colnames(model.frame(object))){
@@ -406,12 +499,12 @@ getSimulations.glmmTMB <- function (object, nsim = 1, type = c("normal", "refit"
         out = as.data.frame(as.matrix(out)[,seq(1, (2*nsim), by = 2)])
       }
     }
-
-
-
-
-
-
+    
+    
+    
+    
+    
+    
     # matrixResp =
     #
     # if(matrixResp & !is.null(ncol(newresp))){
@@ -425,13 +518,13 @@ getSimulations.glmmTMB <- function (object, nsim = 1, type = c("normal", "refit"
     # } else {
     #   newData[[1]] = newresp
     # }
-
-
+    
+    
     # if (out$modelClass == "glmmTMB" & ncol(simulations) == 2*n) simObserved = simulations[,(1+(2*(i-1))):(2+(2*(i-1)))]
   }
-
+  
   # else securityAssertion("Simulation results produced unsupported data structure", stop = TRUE)
-
+  
   return(out)
 }
 
@@ -441,28 +534,28 @@ getSimulations.glmmTMB <- function (object, nsim = 1, type = c("normal", "refit"
 #' @export
 getObservedResponse.HLfit <- function(object, ...){
   out = spaMM::response(object, ...)
-
+  
   nKcase = is.matrix(out)
   if(nKcase){
     if(! (family(object) %in% c("binomial", "betabinomial"))) securityAssertion("nKcase - wrong family")
     if(! (ncol(out)==2)) securityAssertion("nKcase - wrong dimensions of response")
     out = out[,1]
   }
-
+  
   if(!is.numeric(out)) out = as.numeric(out)
-
+  
   return(out)
-
+  
 }
 
 #' @rdname getSimulations
 #' @export
 getSimulations.HLfit <- function(object, nsim = 1, type = c("normal", "refit"), ...){
-
+  
   type <- match.arg(type)
-
+  
   capture.output({out = simulate(object, nsim = nsim, ...)})
-
+  
   if(type == "normal"){
     if(!is.matrix(out)) out = data.matrix(out)
   }else{
@@ -507,8 +600,8 @@ getSimulations.MixMod <- function(object, nsim = 1, type = c("normal", "refit"),
 
 #' @rdname getFixedEffects
 #' @export
-getFixedEffects.MixMod <- function(fittedModel){
-  out <- fixef(fittedModel, sub_model = "main")   
+getFixedEffects.MixMod <- function(object, ...){
+  out <- fixef(object, sub_model = "main")   
   return(out)
 }
 
