@@ -1,8 +1,9 @@
+\dontrun{
 # Loading hurricanes dataset
 
 library(DHARMa)
-data(hurricanes)
 
+data(hurricanes)
 str(hurricanes)
 
 # this is the model fit by Jung et al.
@@ -24,4 +25,5 @@ res2 = recalculateResiduals(res, group = hurricanes$Year)
 testTemporalAutocorrelation(res2, time = unique(hurricanes$Year))
 
 # task: try to address these issues - in many instances, this will 
-# make the MasFem predictor n.s.
\ No newline at end of file
+# make the MasFem predictor n.s.
+}
\ No newline at end of file
